@@ -11,7 +11,7 @@ try:
     import cv2
     import drawsvg as draw
     from datetime import datetime, date
-    import time
+    from distutils import util
 except ImportError as err:
     print("Error: %s" % (err))
 
@@ -27,10 +27,10 @@ class Data:
     def __init__(self) -> None:
         pass
 
-    def add(self, id, name, start, finish, resource, parent):
+    def add(self, id, name, start, finish, resource, next, milestone, parent):
         # duration
-        item = {'id': id, 'name': name, 'begin': start,
-                'end': finish, 'resource': resource}
+        item = {'id': id, 'name': name, 'start': start,
+                'finish': finish, 'resource': resource, 'next': next, 'milestone': eval(milestone)}
         if int(parent) > 0:
             if not 'subitem' in self.data[parent]:
                 self.data[parent]['subitem'] = {}
@@ -56,7 +56,8 @@ class Gantt:
     itemWidth = 30
     barHeight = 20
     progressHeight = 14
-    textSize = 30
+    nameTextSize = 30
+    resourceTextSize = 90
     textIndent = 0
     beginDate = datetime.now().date()
     endDate = datetime.now().date()
@@ -69,7 +70,7 @@ class Gantt:
     def __init__(self) -> None:
 
         self.draw = draw.Drawing(self.canvasWidth, self.canvasHeight)
-        self.draw.append(draw.Rectangle(1, 1, self.canvasWidth - 1,
+        self.draw.append(draw.Rectangle(0, 0, self.canvasWidth - 1,
                                         self.canvasHeight-1, fill='#eeeeee', stroke='black'))
 
     def title(self, text):
@@ -85,21 +86,21 @@ class Gantt:
         group.append(draw.Line(1, 80, self.canvasWidth,
                                80,  stroke='black'))
         group.append(draw.Text('任务', 20, 5, top + 20, fill='#555555'))
-        group.append(draw.Line(self.textSize, top - 30,
-                               self.textSize, self.canvasHeight, stroke='grey'))
-        group.append(draw.Text('开始日期', 20, self.textSize,
+        group.append(draw.Line(self.nameTextSize, top,
+                               self.nameTextSize, self.canvasHeight, stroke='grey'))
+        group.append(draw.Text('开始日期', 20, self.nameTextSize,
                                top + 20, fill='#555555'))
-        group.append(draw.Line(self.textSize + 100, top - 30,
-                               self.textSize + 100, self.canvasHeight, stroke='grey'))
-        group.append(draw.Text('截止日期', 20, self.textSize +
+        group.append(draw.Line(self.nameTextSize + 100, top,
+                               self.nameTextSize + 100, self.canvasHeight, stroke='grey'))
+        group.append(draw.Text('截止日期', 20, self.nameTextSize +
                                100, top + 20, fill='#555555'))
-        group.append(draw.Line(self.textSize + 200, top - 30,
-                               self.textSize + 200, self.canvasHeight, stroke='grey'))
-        group.append(draw.Text('工时', 20, self.textSize +
+        group.append(draw.Line(self.nameTextSize + 200, top,
+                               self.nameTextSize + 200, self.canvasHeight, stroke='grey'))
+        group.append(draw.Text('工时', 20, self.nameTextSize +
                                200, top + 20, fill='#555555'))
-        group.append(draw.Line(self.textSize + 250, top - 30,
-                               self.textSize + 250, self.canvasHeight, stroke='grey'))
-        group.append(draw.Text('资源', 20, self.textSize +
+        group.append(draw.Line(self.nameTextSize + 250, top,
+                               self.nameTextSize + 250, self.canvasHeight, stroke='grey'))
+        group.append(draw.Text('资源', 20, self.nameTextSize +
                                250, top + 20, fill='#555555'))
 
         return group
@@ -190,8 +191,8 @@ class Gantt:
                 weekGroups[weekNumber].append(draw.Line(x + self.unitWidth, top - self.unitHeight,
                                                         x + self.unitWidth, self.canvasHeight, stroke='black'))
 
-            r = draw.Rectangle(x, top, self.unitWidth,
-                               self.canvasHeight, fill=color)
+            r = draw.Rectangle(x, top+1, self.unitWidth,
+                               self.canvasHeight - 80-2, fill=color)
             r.append_title(str(day))
             weekGroups[weekNumber].append(r)
 
@@ -336,16 +337,16 @@ class Gantt:
         left = self.starting
         top = 110 + self.itemLine * self.itemHeight + self.splitLine * self.itemLine
 
-        begin = datetime.strptime(line['begin'], '%Y-%m-%d').day
+        begin = datetime.strptime(line['start'], '%Y-%m-%d').day
         # end = datetime.strptime(line['end'], '%Y-%m-%d').day
-        end = (datetime.strptime(line['end'], '%Y-%m-%d').date() -
-               datetime.strptime(line['begin'], '%Y-%m-%d').date()).days
+        end = (datetime.strptime(line['finish'], '%Y-%m-%d').date() -
+               datetime.strptime(line['start'], '%Y-%m-%d').date()).days
 
         # left += self.itemWidth * (begin - 1) + (1 * begin)
         # # 日宽度 + 竖线宽度
         right = self.itemWidth * (end + 1) + (1 * end)
 
-        left = self.dayPosition[line['begin']]
+        left = self.dayPosition[line['start']]
         # right = self.dayPosition[line['end']]
 
         self.linkPosition[line['id']] = {'x': left, 'y':  top, 'width': right}
@@ -358,23 +359,22 @@ class Gantt:
         # text.append(draw.TSpan(line['end'], text_anchor='start'))
         table.append(text)
         fontSize = self.getTextSize(line['name'])
-        table.append(draw.Text(line['begin'], 20, self.textSize,
+        table.append(draw.Text(line['start'], 20, self.nameTextSize,
                                top + 20, text_anchor='start'))
-        table.append(draw.Text(line['end'], 20, self.textSize +
+        table.append(draw.Text(line['finish'], 20, self.nameTextSize +
                                100, top + 20, text_anchor='start'))
         # if 'progress' in line:
         #     table.append(draw.Text(
-        #         str(line['progress']), 20, self.textSize + 200, top + 20, text_anchor='start'))
+        #         str(line['progress']), 20, self.nameTextSize + 200, top + 20, text_anchor='start'))
 
-        table.append(draw.Text(str(end), 20, self.textSize +
+        table.append(draw.Text(str(end+1), 20, self.nameTextSize +
                      210, top + 20, text_anchor='start'))
         if 'resource' in line:
             table.append(draw.Text(
-                str(line['resource']), 20, self.textSize + 250, top + 20, text_anchor='start'))
+                str(line['resource']), 20, self.nameTextSize + 250, top + 20, text_anchor='start'))
         lineGroup.append(table)
-        group = draw.Group(id='item', fill='none', stroke='black')
-        # text = draw.Text(line['name'], 20, 5, top + 15, text_anchor='start')
-        # group.append(text)
+        group = draw.Group(id='item')
+        # fill='none', stroke='black'
 
         if subitem:
             # print(begin,end)
@@ -398,28 +398,45 @@ class Gantt:
                 left, top + offsetY,
                 fill='black', stroke='black'))
         else:
+            if 'milestone' in line and line['milestone']:
+                mleft = left + 15
+                mtop = top + 4
+                p = draw.Path(fill='black')
+                p.M(mleft, mtop).L(mleft+11, top+15).L(mleft, top +
+                                                       26).L(mleft - 11, top + 15).L(mleft, mtop).Z()
+                group.append(p)
+                group.append(draw.Text(datetime.strptime(line['start'], '%Y-%m-%d').strftime('%m月%d日'),
+                                       18, left + 30, top + 20, text_anchor='start', fill='black'))
+            else:
+                # 工时
+                r = draw.Rectangle(left, top + 4, right,
+                                   self.barHeight, fill='#67AAFF', stroke='black')
+                r.append_title(line['name'])
+                group.append(r)
 
-            # 工时
-            r = draw.Rectangle(left, top + 4, right,
-                               self.barHeight, fill='#ccccff')
-            r.append_title(line['name'])
-            group.append(r)
+                # 进度
+                if 'progress' in line and line['progress'] > 0:
 
-            # 进度
-            if 'progress' in line:
-                progress = draw.Rectangle(
-                    left, top + 7, 30 * line['progress'], self.progressHeight, fill='#ccffff')
-                progress.append_title(str(line['progress']))
-                # mask.append(progress)
-                group.append(progress)
+                    progress = 0
+                    if line['progress'] > end + 1:
+                        progress = end + 1
+                    else:
+                        progress = line['progress']
+
+                    progressBar = draw.Rectangle(
+                        left+2, top + 7, 30 * progress - 2, self.progressHeight, fill='#8AD97A')
+                    # progressBar.append_title(str(progress))
+                    group.append(progressBar)
+                    group.append(draw.Text("%d%%" % ((progress/(end+1))*100),
+                                           10, left + 5, top + 18, text_anchor='start', fill='black'))
 
         # 分割线
         group.append(draw.Lines(1, top + self.itemHeight,
                                 self.canvasWidth, top + self.itemHeight, stroke='grey'))
 
         lineGroup.append(group)
-        self.draw.append(lineGroup)
         self.itemLine += 1
+        return lineGroup
 
     def legend(self):
         top = 10
@@ -428,16 +445,20 @@ class Gantt:
         # print(self.linkPosition)
 
     def task(self):
-        offsetY = 0
+        taskGroup = draw.Group(id='task')
         for id, line in self.data.items():
             if 'subitem' in line:
-                self.items(line, True)
+                item = self.items(line, True)
+                taskGroup.append(item)
                 self.textIndent += 1
                 for id, item in line['subitem'].items():
-                    self.items(item)
+                    item = self.items(item)
+                    taskGroup.append(item)
                 self.textIndent -= 1
             else:
-                self.items(line)
+                item = self.items(line)
+                taskGroup.append(item)
+        self.draw.append(taskGroup)
 
     def link(self, fromTask, toTask):
         # print(fromTask, toTask)
@@ -451,18 +472,22 @@ class Gantt:
                          fill='none', marker_end=arrow)
         path.M(x, y).H(toTask['x']+15).V(toTask['y']-5)
         linkGroup.append(path)
-        self.draw.append(linkGroup)
+        return linkGroup
 
     def next(self):
+        handover = draw.Group(id='handover')
         for id, line in self.data.items():
-            if 'next' in line:
-                self.link(self.linkPosition[line['id']],
-                          self.linkPosition[line['next']])
+            if 'next' in line and line['next'] and int(line['next']) > 0:
+                link = self.link(self.linkPosition[line['id']],
+                                 self.linkPosition[line['next']])
+                handover.append(link)
             if 'subitem' in line:
                 for id, item in line['subitem'].items():
-                    if 'next' in item:
-                        self.link(
+                    if 'next' in item and line['next'] and int(item['next']) > 0:
+                        link = self.link(
                             self.linkPosition[item['id']], self.linkPosition[item['next']])
+                        handover.append(link)
+        self.draw.append(handover)
 
     def workload(self):
         self.starting = 400
@@ -472,13 +497,16 @@ class Gantt:
         for key, value in self.data.items():
             self.fontSize = self.getTextSize(key)
 
-        if self.beginDate > value['start']:
-            self.beginDate = value['start']
+            start = datetime.strptime(value['start'], '%Y-%m-%d').date()
+            finish = datetime.strptime(value['finish'], '%Y-%m-%d').date()
 
-        if self.endDate < value['finish']:
-            self.endDate = value['finish']
+            if self.beginDate > start:
+                self.beginDate = start
 
-        print(self.fontSize)
+            if self.endDate < finish:
+                self.endDate = finish
+
+        # print(self.fontSize)
 
         chart = draw.Group(id='workload')
 
@@ -487,20 +515,20 @@ class Gantt:
         table.append(draw.Line(1, 80, self.canvasWidth,
                                80,  stroke='black'))
         table.append(draw.Text('资源', 20, 5, top + 20, fill='#555555'))
-        table.append(draw.Line(self.textSize + 100, top - 30,
-                     self.textSize + 100, self.canvasHeight, stroke='grey'))
-        table.append(draw.Text('开始日期', 20, self.textSize +
+        table.append(draw.Line(self.nameTextSize + 100, top,
+                     self.nameTextSize + 100, self.canvasHeight, stroke='grey'))
+        table.append(draw.Text('开始日期', 20, self.nameTextSize +
                      100, top + 20, fill='#555555'))
-        table.append(draw.Line(self.textSize + 200, top - 30,
-                     self.textSize + 200, self.canvasHeight, stroke='grey'))
-        table.append(draw.Text('截止日期', 20, self.textSize +
+        table.append(draw.Line(self.nameTextSize + 200, top,
+                     self.nameTextSize + 200, self.canvasHeight, stroke='grey'))
+        table.append(draw.Text('截止日期', 20, self.nameTextSize +
                      200, top + 20, fill='#555555'))
-        table.append(draw.Line(self.textSize + 300, top - 30,
-                     self.textSize + 300, self.canvasHeight, stroke='grey'))
-        table.append(draw.Text('工时', 20, self.textSize +
+        table.append(draw.Line(self.nameTextSize + 300, top,
+                     self.nameTextSize + 300, self.canvasHeight, stroke='grey'))
+        table.append(draw.Text('工时', 20, self.nameTextSize +
                                300, top + 20, fill='#555555'))
-        table.append(draw.Line(self.textSize + 400, top - 30,
-                               self.textSize + 400, self.canvasHeight, stroke='grey'))
+        table.append(draw.Line(self.nameTextSize + 400, top,
+                               self.nameTextSize + 400, self.canvasHeight, stroke='grey'))
 
         chart.append(table)
 
@@ -528,27 +556,27 @@ class Gantt:
         # left += self.itemWidth * (begin - 1) + (1 * begin)
         # # 日宽度 + 竖线宽度
 
-        # left = self.dayPosition[line['begin']]
-
         for resource, row in self.data.items():
             #     print(resource, row)
 
             # # 工时
             top = 110 + self.itemLine * self.itemHeight + self.splitLine * self.itemLine
-            # end = (datetime.strptime(line['end'], '%Y-%m-%d').date() -               datetime.strptime(line['begin'], '%Y-%m-%d').date()).days
-            end = (row['finish'] - row['start']).days
+            end = (datetime.strptime(row['finish'], '%Y-%m-%d').date() -
+                   datetime.strptime(row['start'], '%Y-%m-%d').date()).days
+            # end = (row['finish'] - row['start']).days
             right = self.itemWidth * (end + 1) + (1 * end)
 
             chart.append(draw.Text(resource, 20, 5 + (self.textIndent *
                          self.itemWidth), top + 20, text_anchor='start'))
-            chart.append(draw.Text(row['start'].strftime('%Y-%m-%d'), 20, self.textSize + 100,
+            chart.append(draw.Text(row['start'], 20, self.nameTextSize + 100,
                                    top + 20, text_anchor='start'))
-            chart.append(draw.Text(row['finish'].strftime('%Y-%m-%d'), 20, self.textSize +
+            chart.append(draw.Text(row['finish'], 20, self.nameTextSize +
                                    200, top + 20, text_anchor='start'))
 
-            chart.append(draw.Text(str(end), 20, self.textSize +
+            chart.append(draw.Text(str(end), 20, self.nameTextSize +
                                    300, top + 20, text_anchor='start'))
 
+            left = self.dayPosition[row['start']]
             r = draw.Rectangle(left, top + 4, right,
                                self.barHeight, fill='#aaaaaa')
             r.append_title(resource)
@@ -579,29 +607,35 @@ class Gantt:
         length = self.getTextSize(item['name'])
         # print(item['name'], length)
         # 文本表格所占用的宽度
-        if self.textSize < length - 50:
-            self.textSize = length - 50
+        if self.nameTextSize < length - 50:
+            self.nameTextSize = length - 50
             # print(item['name'], len(item['name']))
 
-        begin = datetime.strptime(item['begin'], '%Y-%m-%d').date()
+        if 'resource' in item:
+            length = self.getTextSize(item['resource'])
+            if self.resourceTextSize < length - 50:
+                self.resourceTextSize = length - 50
+
+        begin = datetime.strptime(item['start'], '%Y-%m-%d').date()
         if self.beginDate > begin:
             self.beginDate = begin
 
-        end = datetime.strptime(item['end'], '%Y-%m-%d').date()
+        end = datetime.strptime(item['finish'], '%Y-%m-%d').date()
         if self.endDate < end:
             self.endDate = end
         # print(self.endDate)
 
     def ganttChart(self):
-
+        textIndent = 0
         for id, line in self.data.items():
             self.initialize(line)
             if 'subitem' in line:
                 for id, item in line['subitem'].items():
                     self.initialize(item)
+                textIndent = 30
 
-        self.starting = self.textSize + 310
-     # print(self.starting, self.textSize)
+        self.nameTextSize += textIndent
+        self.starting = self.nameTextSize + self.resourceTextSize + 250
 
         self.background()
         self.task()
